//******************************************************************************
// "Evaluating the Macroeconomic Effects of the ECB’s Unconventional Monetary Policy "
//
// By Sarah Mouabbi and Jean-Guillaume Sahuc
//
// Estimation
//
//******************************************************************************


//****************************************
//******************************
// Model's Description 
//******************************
//****************************************

var dyobs dcobs dxobs dwobs inflobs srobs hoursobs
y c x w infl r labor k u kp lambda q rk mc g mrs
ez eb ex ep eg ew armap armaw;

varexo zetaz zetab zetax zetap zetag zetar zetaw;

parameters psiu etak hab nu thetaw thetap gammaw gammap epbar ewbar rhoinfl rhoy rhoz rhow rhob rhox rhop rhog rhor rhop2 rhow2
consteinfl constelabor delta alpha gsy conster gam beta sigma;

//******************************
// Fixed parameters
//******************************

beta  = 0.998;
alpha = 0.30;
delta = 0.025;
gsy   = 0.20;
ewbar = 1.1;
epbar = 1.2;
rhoz  = 0;
sigma = 1;


model(linear); 

//******************************
// Steady-state
//******************************

#inflbar = 1 + consteinfl/100;
#gammaz = exp(gam/100);
#rbar   = (gammaz^sigma)*inflbar/beta;
#rkbar  = (gammaz^sigma)/beta - (1-delta);
#wbar   = (1/epbar*(alpha^alpha)*((1-alpha)^(1-alpha))/(rkbar^alpha))^(1/(1-alpha));
#ksl    = wbar/rkbar*alpha/(1-alpha);
#fsl    = ksl^alpha - wbar - rkbar*ksl;
#ysl    = ksl^alpha-fsl;
#fsy    = fsl/ysl;
#xsl    = (1-(1-delta)/gammaz)*gammaz*ksl;
#xsy    = xsl/ysl;
#csy    = 1 - xsy - gsy;
#ksy    = ksl/ysl;
#kappap  = (1-beta*thetap)*(1-thetap)/(thetap*(1+beta*gammap));
#kappaw = (1-beta*thetaw)*(1-thetaw)/(thetaw*(1+beta)*(1+nu*(ewbar/(ewbar-1))));
#etau   = (1-psiu)/psiu;

//***********************************
// Dynamics
//***********************************

// Effective capital
k  = u + kp(-1) - ez;

// Physical capital accumulation
kp = ((1-delta)/gammaz)*(kp(-1)-ez) + (1-(1-delta)/gammaz)*(x + (etak*gammaz^2*(1+beta))*ex);

// Marginal utility of consumption
lambda = hab*beta*(gammaz^(-sigma))*sigma*gammaz/((1-hab*beta*(gammaz^(-sigma)))*(gammaz-hab))*c(+1) - sigma*(gammaz+hab^2*beta*(gammaz^(-sigma)))/((1-hab*beta*(gammaz^(-sigma)))*(gammaz-hab))*c
+ sigma*hab/((1-hab*beta*(gammaz^(-sigma)))*(gammaz-hab))*c(-1) + (hab*sigma*beta*gammaz^(-sigma+1)*rhoz-hab*sigma)/((1-hab*beta*(gammaz^(-sigma)))*(gammaz-hab))*ez
+ (1-hab*beta*gammaz^(-sigma)*rhob)/(1-hab*beta*gammaz^(-sigma))*eb;


//Euler equation
lambda = lambda(+1) + (r-infl(+1)) - sigma*ez(+1);

// Investment
x = 1/(1+beta)*(x(-1)-ez) + beta/(1+beta)*(x(+1)+ez(+1)) + 1/(etak*gammaz^2*(1+beta))*q  + ex;

// Tobin's Q
q = beta*(1-delta)/gammaz*q(+1) + (1-beta*(1-delta)/gammaz)*rk(+1) - (r - infl(+1));

// Capital utilization
u = etau*rk;

// Production function
y = (1+fsy)*(alpha*k + (1-alpha)*labor);

// Labor demand
w = mc + alpha*k - alpha*labor;

// Capital renting
rk = mc - (1-alpha)*k + (1-alpha)*labor;

// Phillips curve
infl = gammap/(1+gammap*beta)*infl(-1) + beta/(1+gammap*beta)*infl(+1) + kappap*mc + ep;

// Wage curve
w = 1/(1+beta)*w(-1) + beta/(1+beta)*w(+1) - kappaw*mrs 
+ gammaw/(1+beta)*infl(-1) - (1+beta*gammaw)/(1+beta)*infl + beta/(1+beta)*infl(+1)
- (1-beta*rhoz)/(1+beta)*ez + ew ;

// Marginal Rate of Substitution
mrs = w - (nu*labor - lambda + eb);

// Government spending
g = y + (1-gsy)/gsy*eg;

// Monetary policy
r = rhor*r(-1) + (1-rhor)*(rhoinfl*infl + rhoy*(y-y(-1)+ez)) + zetar ;

//Resource constraint
y = csy*c + xsy*x + gsy*g + rkbar*ksy*u;

// Shocks processes
ez    = rhoz*ez(-1) + zetaz;
eb    = rhob*eb(-1) + zetab;
ex    = rhox*ex(-1) + zetax;
eg    = rhog*eg(-1) + zetag;
ep    = rhop*ep(-1) + armap -rhop2*armap(-1);
armap = zetap;
ew    = rhow*ew(-1) + armaw -rhow2*armaw(-1);
armaw = zetaw;

// Measurement equations

dyobs   = y-y(-1) + gam + ez;
dcobs   = c-c(-1) + gam + ez;
dxobs   = x-x(-1) + gam + ez;
dwobs   = w-w(-1) + gam + ez;
inflobs  = infl + consteinfl;
srobs    = r + conster;
hoursobs = labor + constelabor;

end; 

shocks;
var zetaz;
stderr 0.;
var zetab;
stderr 0.;
var zetax;
stderr 0.;
var zetap;
stderr 0.;
var zetag;
stderr 0.;
var zetar;
stderr 0.;
var zetaw;
stderr 0.;
end;

estimated_params;

stderr zetaz,   inv_gamma_pdf, 0.25, 2;
stderr zetab,   inv_gamma_pdf, 0.25, 2;
stderr zetax,   inv_gamma_pdf, 0.25, 2;
stderr zetap,   inv_gamma_pdf, 0.25, 2;
stderr zetag,   inv_gamma_pdf, 0.25, 2;
stderr zetar,   inv_gamma_pdf, 0.25, 2;
stderr zetaw,   inv_gamma_pdf, 0.25, 2;

gam,         normal_pdf, 0.30, 0.05;
psiu,        beta_pdf,   0.50, 0.10;
etak,        gamma_pdf,  4.00, 1.00;
hab,         beta_pdf,   0.70, 0.10;
nu,          gamma_pdf,  2.00, 0.75;
thetaw,      beta_pdf,   0.66, 0.10;
thetap,      beta_pdf,   0.66, 0.10;
gammaw,      beta_pdf,   0.50, 0.15;
gammap,      beta_pdf,   0.50, 0.15;
rhoinfl,     gamma_pdf,  1.70, 0.30;
rhoy,        gamma_pdf,  0.125, 0.05;
rhor,        beta_pdf,   0.75, 0.15;
rhob,        beta_pdf,   0.50, 0.20;
rhox,        beta_pdf,   0.50, 0.20;
rhop,        beta_pdf,   0.50, 0.20;
rhop2,       beta_pdf,   0.50, 0.20;
rhog,        beta_pdf,   0.50, 0.20;
rhow,        beta_pdf,   0.50, 0.20;
rhow2,       beta_pdf,   0.50, 0.20;
constelabor, normal_pdf, -1.37, 0.10;
consteinfl,  gamma_pdf,  0.40, 0.15;
conster,     gamma_pdf,  0.35, 0.15;
end;

varobs dyobs dcobs dxobs dwobs inflobs srobs hoursobs;

estimation(datafile=us_data_17Q2,first_obs=1, order=1,mh_replic=500000,mh_nblocks=2, mh_jscale=0.35,mh_drop=0.25,plot_priors=0);